#!/usr/bin/env python3

from optparse import OptionParser
import sys, getopt
import os
import re

remote_repo = 'git@github.com:sendbird/sendbird-sdk-flutter'
public_repo = '../sendbird-sdk-flutter'
cloud_url = 'https://api.cloudsmith.io/v1/'
api_key = '7582c4db163524e634f19db2ecfde507b67b1fae'

def run_test():
  print("[SDK] Running unit tests...")
  try:
    os.system("rm -rf *.info")
    os.system("./report")
    os.system("./report --report")
  except:
    return False

  coverage = 0.0
  pattern = re.compile("<td class=\"headerCovTableEntryMed\">(.*) %</td>")
  for i, line in enumerate(open('./coverage/index.html')):
    for match in re.finditer(pattern, line):
      coverage = float(match.group(1))

  print("[SDK] test coverage {0}".format(coverage))  
  return coverage > 80.0

def copy_files():
  print("[SDK] Running copy files into destination...")
  global public_repo, remote_repo
  
  cwd = os.getcwd()
    
  if not os.path.isdir(public_repo):
    os.chdir('../')
    public_repo = os.getcwd() + '/sendbird-sdk-flutter'
    os.system("git clone {0}".format(remote_repo))
    os.chdir(cwd)
   
  dest = public_repo
  
  try:
    os.system("rm -r {0}/*".format(dest))
    os.system("cp -r lib {0}".format(dest))
    os.system("cp -r example {0}".format(dest))
    os.system("cp CHANGELOG.md {0}/CHANGELOG.md".format(dest))
    os.system("cp README.md {0}/README.md".format(dest))
    os.system("cp build.yaml {0}/build.yaml".format(dest))
    os.system("cp pubspec.yaml {0}/pubspec.yaml".format(dest))
    os.system("cp pubspec.lock {0}/pubspec.lock".format(dest))
    os.system("cp analysis_options.yaml {0}/analysis_options.yaml".format(dest))
    os.system("cp LICENSE {0}/LICENSE".format(dest))
  except Exception as err:
    print("[SDK] failed to copy into destination {0}".format(err))
    sys.exit(1)

def update_repo(version, message, forced):
  print("[SDK] Running update on git repository...")
  global public_repo

  if message is None:
    message = "Release version {0}".format(version)

  try:
    os.chdir(public_repo)
    os.system("git add .")
 
    if forced:
      os.system("git commit --amend -m \"{0}\"".format(message))
      os.system("git push --force")
      os.system("git tag -d v{0}".format(version))
      os.system("git push origin :refs/tags/v{0}".format(version))
    else:
      os.system("git commit -m \"{0}\"".format(message))
      os.system("git push")
    os.system("git tag -a v{0} -m \"\"".format(version))
    os.system("git push --tags")
  except Exception as e:
    print("[SDK] failed to update git repo: {0}")
    sys.exit(1)

def compress(version):
  print("[SDK] Running compression project folder...")
  global public_repo
  
  try:
    os.chdir(public_repo)
    pkg_version = version.replace('.', '_')
    compressed_file_name = "sdk_{0}.tar.gz".format(pkg_version)
    os.system("tar --exclude='.dart_tool' --exclude='.git' -czvf {0} -C {1} .".format(compressed_file_name, public_repo))
  except Exception as e:
    print("[SDK] failed to compress the project: {0}".format(e))
    sys.exit(1)

  return compressed_file_name

def deploy(filename, forced):
  print("[SDK] Uploading compressed file to cloud...")
  global api_key, public_repo
  
  arg = "-k {0} sendbird/release {1}".format(api_key, filename)
  if forced:
    arg += " --republish"

  try:
    os.chdir(public_repo)
    os.system("cloudsmith push dart {0}".format(arg))
    os.system("rm {0}".format(filename))
  except Exception as e:
    print("[SDK] failed to upload zip to cloud: {0}".format(e))
    sys.exit(1)

def create_doc():
  try:
    os.system("dartdoc")
  except Exception as e:
    sys.exit(1)

def parse_args(argv):
  parser = OptionParser("usage: %prog -d <dest_repo_path> -v <version>")
  parser.add_option("-v", "--version", dest="version", help="release version")
  parser.add_option("-m", "--message", dest="message", action="store",  help="commit message")
  parser.add_option("-f", "--force_push", dest="forced", action="store_true", default=False, help="force push to repository")
  parser.add_option("-p", "--push", dest="push", action="store_true", default=False, help="push only")
  parser.add_option("-c", "--copy", dest="copy", action="store_true", default=False, help="copy repository")
  (options, args) = parser.parse_args()
 
  if options.version is None:
    parser.error("release version is required")
 
  try:
    os.system("which cloudsmith")
  except:
    print("[SDK] required to install cloudsmith-cli via pip")
    sys.exit(1)
    
  return options

if __name__ == "__main__":
  options = parse_args(sys.argv[1:])

  version = options.version
  msg = options.message
  forced = options.forced
  copy = options.copy

  #if options.copy is not None:
  #  copy_files()
  #  sys.exit(0)

  #if options.push is not None:
  #  ouptut = compress(version)
  #  deploy(output, forced)
  #  sys.exit(0)

  #if not run_test():
  #  sys.exit(1)

  # copy files from private repo to public repo
  copy_files()
  
  # update public repo with push branch and tag
  update_repo(version, msg, forced)
 
  #compress public repo
  #output = compress(version)

  #deploy into cloud 
  #deploy(output, forced)

