import 'package:flutter_test/flutter_test.dart';
import 'package:sendbird_sdk/sendbird_sdk.dart';

import '../../common/helpers/private_api_client.dart';
import '../../common/request.dart';
import '../../common/sdksetup.dart';
import '../../common/user.dart';

void main() {
  late TestAppInfo testInfo;
  late SendbirdSdk sdk;

  late EmojiCategory pc;
  late EmojiCategory tc;
  late EmojiCategory fc;

  setUpAll(() async {
    testInfo = await testCreateApp();

    //create category
    pc = await PrivateApiClient().send(AppEmojiCategoryCreateRequest(
      categoryName: 'person',
      categoryUrl: 'http://google.com',
    ));
    tc = await PrivateApiClient().send(AppEmojiCategoryCreateRequest(
      categoryName: 'things',
      categoryUrl: 'http://google.com',
    ));
    fc = await PrivateApiClient().send(AppEmojiCategoryCreateRequest(
      categoryName: 'food',
      categoryUrl: 'http://google.com',
    ));

    //create emojis
    await PrivateApiClient().send(AppEmojiCreateRequest(
      categoryId: pc.id,
      key: 'happy_person',
      emojiUrl: 'http://google.com',
    ));

    await PrivateApiClient().send(AppEmojiCreateRequest(
      categoryId: pc.id,
      key: 'sad_person',
      emojiUrl: 'http://google.com',
    ));

    await PrivateApiClient().send(AppEmojiCreateRequest(
      categoryId: pc.id,
      key: 'hands_on_person',
      emojiUrl: 'http://google.com',
    ));

    await PrivateApiClient().send(AppEmojiCreateRequest(
      categoryId: tc.id,
      key: 'top_building',
      emojiUrl: 'http://google.com',
    ));

    await PrivateApiClient().send(AppEmojiCreateRequest(
      categoryId: tc.id,
      key: 'fat_building',
      emojiUrl: 'http://google.com',
    ));

    await PrivateApiClient().send(AppEmojiCreateRequest(
      categoryId: fc.id,
      key: 'apple',
      emojiUrl: 'http://google.com',
    ));
  });

  setUp(() async {
    sdk = await testApiSdkSetup(testInfo: testInfo);
    await testApiCreateUserAndConnect();
  });

  tearDown(() async {
    await sdk.disconnect();
  });

  group('emoji get', () {
    test('should success to get an emoji', () async {
      final emoji = await sdk.getEmoji('sad_person');
      expect(emoji, isNotNull);
      expect(emoji.key, equals('sad_person'));
      expect(emoji.url, equals('http://google.com'));
    });

    test('should throw an exception with invalid key', () async {
      expect(() => sdk.getEmoji('fake'), throwsA(isA<SBError>()));
    });
  });

  group('emoji get category', () {
    test('should success to get category', () async {
      final category = await sdk.getEmojiCategory(pc.id);
      expect(category, isNotNull);
      expect(category.emojis, isNotNull);
      expect(category.emojis.length, equals(3));
    });

    test('should throw an expcetion with invalid id', () async {
      expect(() => sdk.getEmojiCategory(0), throwsA(isA<SBError>()));
    });
  });

  group('emoji get all', () {
    test('should success to return values', () async {
      final emojiContainer = await sdk.getAllEmojis();
      expect(emojiContainer, isNotNull);
      expect(emojiContainer.emojiHash, isNotNull);
      // includes default category sendbird
      expect(emojiContainer.emojiCategories.length, equals(4));
    });

    test('should identical to current app info emoji hash', () async {
      final emojiContainer = await sdk.getAllEmojis();
      expect(
        sdk.getAppInfo()?.isEmojiUpdatedNeeded(emojiContainer.emojiHash),
        isFalse,
      );
    });
  });
}
