import 'dart:async';

import 'package:sendbird_sdk/sendbird_sdk.dart';
import 'package:test/test.dart';

import '../../common/callback.dart';
import '../../common/common.dart';
import '../../common/helpers/private_api_client.dart';
import '../../common/request.dart';
import '../../common/sdksetup.dart';
import '../../common/user.dart';

void main() {
  late TestAppInfo testInfo;
  late SendbirdSdk sdk;
  late ChannelEventListener event;
  late GroupChannel channel;
  late User user;

  final url =
      'https://static.sendbird.com/test-resources/og-tag/og_tag_sample.html';

  setUpAll(() async {
    testInfo = await testCreateApp();
  });

  setUp(() async {
    sdk = await testApiSdkSetup(testInfo: testInfo);
    user = await testApiCreateUserAndConnect();
    event = ChannelEventListener();

    final params = GroupChannelParams()
      ..operatorUserIds = [user.userId]
      ..userIds = [user.userId];
    channel = await createGroupChannel(params);
  });

  tearDown(() async {
    await sdk.disconnect();
  });

  tearDownAll(() async {
    //delete app?
  });

  group('og meta data', () {
    test('should include in updated message when no cache', () async {
      await PrivateApiClient().send(AppOGTagClearRequest([url]));

      final mParams = UserMessageParams(message: url);
      final pending = channel.sendUserMessage(
        mParams,
        onCompleted: (message, error) {
          expect(message, isNotNull);
          expect(message.messageId, isNotNull);
          expect(message.sendingStatus, MessageSendingStatus.succeeded);
        },
      );

      expect(pending, isNotNull);
      expect(pending.sendingStatus, MessageSendingStatus.pending);

      final results = await event.waitFor([
        EventFuncName.onChannelChanged,
        EventFuncName.onMessageUpdated,
      ]);

      expect(results, isNotNull);
      expect(results.length, equals(2));
    });

    test('should included in response when cached', () async {
      await PrivateApiClient().send(AppOGTagClearRequest([url]));

      final mParams = UserMessageParams(message: url);
      final pending = channel.sendUserMessage(
        mParams,
        onCompleted: (message, error) {
          expect(message, isNotNull);
          expect(message.messageId, isNotNull);
          expect(message.sendingStatus, MessageSendingStatus.succeeded);
        },
      );

      expect(pending, isNotNull);
      expect(pending.sendingStatus, MessageSendingStatus.pending);

      final results = await event.waitFor([
        EventFuncName.onChannelChanged,
        EventFuncName.onMessageUpdated,
      ]);

      expect(results, isNotNull);
      expect(results.length, equals(2));

      final wait = Completer();

      channel.sendUserMessage(
        mParams,
        onCompleted: (message, error) {
          expect(message, isNotNull);
          expect(message.messageId, isNotNull);
          expect(message.sendingStatus, MessageSendingStatus.succeeded);
          expect(message.ogMetaData, isNotNull);
          expect(message.ogMetaData?.defaultImage, isNotNull);
          wait.complete();
        },
      );

      await wait.future;
    });
  });
}
