import 'package:sendbird_sdk/request/feature/group_channel_delivery_request.dart';
import 'package:sendbird_sdk/sendbird_sdk.dart';
import 'package:test/test.dart';

import '../../../common/callback.dart';
import '../../../common/common.dart';
import '../../../common/helpers/private_api_client.dart';
import '../../../common/sdksetup.dart';
import '../../../common/user.dart';

void main() {
  late TestAppInfo testInfo;
  late SendbirdSdk sdk;

  late ChannelEventListener event;
  late GroupChannel channel;
  late User user;
  var users = <User>[];

  setUpAll(() async {
    testInfo = await testCreateApp();
  });

  setUp(() async {
    sdk = await testApiSdkSetup(testInfo: testInfo);
  });

  tearDown(() async {
    await sdk.disconnect();
  });

  tearDownAll(() async {});

  group('groupchannel delivery receipt', () {
    setUp(() async {
      users = await testApiCreateUsers(4);
      user = await testApiConnect(users.first.userId);
      event = ChannelEventListener();
      final params = GroupChannelParams()
        ..operatorUserIds = [user.userId]
        ..userIds = users.map((e) => e.userId).toList();
      channel = await createGroupChannel(params);
    });

    test('should get updated my receipt when receving message', () async {
      final params = UserMessageParams(message: 'delivered');
      final msg = await sendUserMessageViaAPI(
        channel: channel,
        params: params,
        senderId: users[1].userId,
      );

      expect(channel.lastMessage?.messageId, equals(msg.messageId));
      expect(channel.memberCount, equals(4));
      //exclude sender and current user
      expect(channel.getUndeliveredMembers(msg).length, equals(2));
    });

    test('should return updated members after reconnect', () async {
      final params = UserMessageParams(message: 'message 1');
      final msg = await sendUserMessageViaAPI(
        channel: channel,
        params: params,
        senderId: users[1].userId,
      );
      expect(channel.getUndeliveredMembers(msg).length, equals(2));

      await sdk.disconnect();

      final msg2 = await sendUserMessageViaAPI(
        channel: channel,
        params: params..message = 'message 2',
        senderId: users.last.userId,
      );

      await PrivateApiClient().send(GroupChannelMarkAsDeliveredRequest(
        channelUrl: channel.channelUrl,
        userId: users[2].userId,
        timestamp: DateTime.now().millisecondsSinceEpoch,
      ));

      await testApiConnect(user.userId);
      event = ChannelEventListener();
      channel = await GroupChannel.getChannel(channel.channelUrl);
      expect(channel.getUndeliveredMembers(msg2).length, equals(1));

      await PrivateApiClient().send(GroupChannelMarkAsDeliveredRequest(
        channelUrl: channel.channelUrl,
        userId: users[1].userId,
        timestamp: DateTime.now().millisecondsSinceEpoch,
      ));

      await event.waitFor([EventFuncName.onDeliveryReceiptUpdated]);
      expect(channel.getUndeliveredMembers(msg2).length, equals(0));
    });
  });

  group('groupchannel undelivered member count', () {
    setUp(() async {
      users = await testApiCreateUsers(4);
      user = await testApiConnect(users.first.userId);
      event = ChannelEventListener();
      final params = GroupChannelParams()
        ..operatorUserIds = [user.userId]
        ..userIds = [user.userId];
      channel = await createGroupChannel(params);
    });

    test('should return non zero after receive a message', () async {
      final params = UserMessageParams(message: 'text');
      channel.sendUserMessage(params);
      await event.waitFor([EventFuncName.onChannelChanged]);
      //in channel alone so count is 0
      expect(
        channel.getUndeliveredMembers(channel.lastMessage!).length,
        equals(0),
      );

      channel.inviteUsers([users.last.userId]);
      await event.waitFor([
        EventFuncName.onUserReceivedInvitation,
        EventFuncName.onUserJoined,
      ]);

      //newly joined member should not be affected by previous messages
      expect(
        channel.getUndeliveredMembers(channel.lastMessage!).length,
        equals(0),
      );
    });
  });
}
