import 'package:flutter_test/flutter_test.dart';
import 'package:sendbird_sdk/request/messages/user_message_send_request.dart';
import 'package:sendbird_sdk/sendbird_sdk.dart';

import '../../../common/callback.dart';
import '../../../common/common.dart';
import '../../../common/helpers/private_api_client.dart';
import '../../../common/sdksetup.dart';
import '../../../common/user.dart';

void main() {
  late TestAppInfo testInfo;
  late SendbirdSdk sdk;
  late ChannelEventListener event;
  late GroupChannel channel;
  late User user;
  late List<User> users;

  setUpAll(() async {
    testInfo = await testCreateApp();
  });

  setUp(() async {
    sdk = await testApiSdkSetup(testInfo: testInfo);
    users = await testApiCreateUsers(2);
    user = await testApiConnect(users.first.userId);

    event = ChannelEventListener();

    final params = GroupChannelParams()
      ..operatorUserIds = [user.userId]
      ..userIds = users.map((e) => e.userId).toList();
    channel = await createGroupChannel(params);
  });

  tearDown(() async {
    await sdk.disconnect();
  });

  tearDownAll(() async {});

  group('groupchannel report channel', () {
    test('should success with valid params', () async {
      await channel.reportChannel(
        category: ReportCategory.harassing,
        description: 'harassing!!',
      );
    });
  });

  group('groupchannel report channel message', () {
    test('should success reporting message with valid params', () async {
      await PrivateApiClient().send(ChannelUserMessageSendApiRequest(
        channelType: channel.channelType,
        channelUrl: channel.channelUrl,
        params: UserMessageParams(message: 'miss you'),
        senderId: users.last.userId,
      ));

      final result = await event.waitFor([EventFuncName.onChannelChanged]);
      final message = (result.first.data.first as GroupChannel).lastMessage!;
      await channel.reportMessage(
          message: message,
          category: ReportCategory.inappropriate,
          description: 'channel report description');
    });

    test('should throw an exception when sendId is empty', () async {
      await PrivateApiClient().send(ChannelUserMessageSendApiRequest(
        channelType: channel.channelType,
        channelUrl: channel.channelUrl,
        params: UserMessageParams(message: 'miss you'),
        senderId: users.last.userId,
      ));

      final result = await event.waitFor([EventFuncName.onChannelChanged]);
      final message = (result.first.data.first as GroupChannel).lastMessage!;
      message.sender = null;
      expect(
          () => channel.reportMessage(
              message: message,
              category: ReportCategory.inappropriate,
              description: 'channel report description'),
          throwsA(isA<InvalidParameterError>()));
    });
  });

  group('groupchannel report user', () {
    test('should success reporting user with valid params', () async {
      await channel.reportUser(
        userId: users.last.userId,
        category: ReportCategory.spam,
        description: 'channel report description',
      );
    });
  });
}
