import 'package:sendbird_sdk/sendbird_sdk.dart';
import 'package:test/test.dart';

import '../../../common/helpers/private_api_client.dart';
import '../../../common/sdksetup.dart';
import '../../../common/user.dart';
import '../../../common/common.dart';

void main() {
  late TestAppInfo testInfo;
  late SendbirdSdk sdk;
  late GroupChannel channel;
  late User user;

  setUpAll(() async {
    testInfo = await testCreateApp();
  });

  setUp(() async {
    sdk = await testApiSdkSetup(testInfo: testInfo);
    user = await testApiCreateUserAndConnect();

    final params = GroupChannelParams()
      ..operatorUserIds = [user.userId]
      ..userIds = [user.userId];
    channel = await createGroupChannel(params);
  });

  tearDown(() async {
    await sdk.disconnect();
  });

  tearDownAll(() async {
    //delete app?
  });

  group('groupchannel send admin message', () {
    test('should succeed with given valud admin message', () async {
      final mesg = await PrivateApiClient().sendAdminMessage(
          channelType: channel.channelType,
          channelURL: channel.channelUrl,
          message: 'this is admin message');

      expect(mesg, isA<AdminMessage>());
      expect(mesg.message, equals('this is admin message'));
    });

    test('should return admin message object toJson', () async {
      final mesg = await PrivateApiClient().sendAdminMessage(
          channelType: channel.channelType,
          channelURL: channel.channelUrl,
          message: 'this is admin message');

      final json = mesg.toJson();
      expect(json, isNotNull);
      expect(json, isA<Map<String, dynamic>>());
      expect(json['channel_type'], equals('group'));
      expect(json['message'], equals('this is admin message'));
    });

    test('should throw an exception when copying message', () async {
      final mesg = await PrivateApiClient().sendAdminMessage(
          channelType: channel.channelType,
          channelURL: channel.channelUrl,
          message: 'this is admin message');

      expect(() => channel.copyMessage(mesg, channel),
          throwsA(isA<InvalidParameterError>()));
    });
  });
}
