//  Copyright © 2020 SENDBIRD.COM. All rights reserved.

import 'package:sendbird_sdk/sendbird_sdk.dart';

import '../../../common/user.dart';
import '../../../common/sdksetup.dart';

import 'package:test/test.dart';

void main() {
  late TestAppInfo testInfo;
  late SendbirdSdk sdk;

  setUpAll(() async {
    testInfo = await testCreateApp();
  });

  setUp(() async {
    sdk = await testApiSdkSetup(testInfo: testInfo);
  });

  tearDown(() async {
    await sdk.disconnect();
  });

  tearDownAll(() async {
    //delete app?
  });

  group('openchannel enter', () {
    test('should be able to enter a valid channel', () async {
      final user = await testApiCreateUserAndConnect();

      final params = OpenChannelParams()
        ..name = 'channel'
        ..operatorUserIds = [user.userId];

      final channel = await OpenChannel.createChannel(params);
      await channel.enter();

      expect(channel.participantCount, equals(1));
      expect(channel.operators.length, equals(1));
      expect(channel.operators.first.userId, equals(user.userId));
      final key = channel.key;
      expect(key, isNotNull);
      expect(key, isA<String>());
    });

    test('should throw exception when no connection', () async {
      final user = await testApiCreateUserAndConnect();

      final params = OpenChannelParams()
        ..name = 'channel'
        ..operatorUserIds = [user.userId];

      final channel = await OpenChannel.createChannel(params);
      await sdk.disconnect();

      expect(channel.enter(), throwsA(isA<ConnectionRequiredError>()));
    });
  });

  group('openchannel exit', () {
    test('should be able to exit entered channel', () async {
      final user = await testApiCreateUserAndConnect();

      final params = OpenChannelParams()
        ..name = 'channel'
        ..operatorUserIds = [user.userId];

      final channel = await OpenChannel.createChannel(params);

      await channel.enter();
      await channel.exit();

      expect(channel.participantCount, equals(0));
    });

    test('should not throw exception when trying to exit not entered channel',
        () async {
      final user = await testApiCreateUserAndConnect();

      final params = OpenChannelParams()
        ..name = 'channel'
        ..operatorUserIds = [user.userId];

      final channel = await OpenChannel.createChannel(params);
      await channel.exit();
    });
  });

  group('openchannel isOperator', () {
    test('should return true if a user is an operator', () async {
      final user = await testApiCreateUserAndConnect();
      final params = OpenChannelParams()
        ..name = 'channel'
        ..operatorUserIds = [user.userId];

      final channel = await OpenChannel.createChannel(params);
      expect(channel.isOperator(user.userId), isTrue);
    });

    test('should return false if a user is not an operator', () async {
      final user = await testApiCreateUserAndConnect();
      final other = await testApiCreateUsers(1);
      final params = OpenChannelParams()
        ..name = 'channel'
        ..operatorUserIds = [user.userId];

      final channel = await OpenChannel.createChannel(params);
      expect(channel.isOperator(other.first.userId), isFalse);
    });
  });
}
