import 'package:sendbird_sdk/request/messages/user_message_send_request.dart';
import 'package:sendbird_sdk/sendbird_sdk.dart';
import 'package:test/test.dart';

import '../../../common/callback.dart';
import '../../../common/common.dart';
import '../../../common/helpers/private_api_client.dart';
import '../../../common/sdksetup.dart';
import '../../../common/user.dart';

void main() {
  late TestAppInfo testInfo;
  late SendbirdSdk sdk;
  late ChannelEventListener event;
  late OpenChannel channel;
  late User user;
  late List<User> users;

  setUpAll(() async {
    testInfo = await testCreateApp();
  });

  setUp(() async {
    sdk = await testApiSdkSetup(testInfo: testInfo);
    users = await testApiCreateUsers(2);
    user = await testApiConnect(users.first.userId);
    event = ChannelEventListener();

    final params = OpenChannelParams()..operatorUserIds = [user.userId];
    channel = await createOpenChannel(params: params, user: user);
  });

  tearDown(() async {
    await sdk.disconnect();
  });

  tearDownAll(() async {});

  group('openchannel report channel', () {
    test('should success with valid params', () async {
      await channel.reportChannel(
        category: ReportCategory.harassing,
        description: 'harassing!!',
      );
    });
  });

  group('openchannel report channel message', () {
    test('should success reporting message with valid params', () async {
      await PrivateApiClient().send(ChannelUserMessageSendApiRequest(
        channelType: channel.channelType,
        channelUrl: channel.channelUrl,
        params: UserMessageParams(message: 'fuck you'),
        senderId: users.last.userId,
      ));

      final result = await event.waitFor([EventFuncName.onMessageReceived]);
      final message = result.first.data.last as UserMessage;
      await channel.reportMessage(
        message: message,
        category: ReportCategory.inappropriate,
      );
    });
  });

  group('openchannel report user', () {
    test('should success reporting user with valid params', () async {
      await channel.reportUser(
          userId: users.last.userId, category: ReportCategory.spam);
    });
  });
}
