import 'package:universal_io/io.dart';
import 'package:flutter/foundation.dart' show kIsWeb;

enum EnvironmentType { local, staging, nightly, production }

class Environment {
  EnvironmentType type;
  late String _token;

  Environment({required this.type});

  String? getApiUrl() {
    var apiHost = const String.fromEnvironment('API_HOST');
    if (apiHost.isEmpty && kIsWeb) {
      throw Exception(
          'You must define API_HOST. This can be done with the --dart-define arg to run or build');
    }
    return kIsWeb ? apiHost : Platform.environment['API_HOST'];
  }

  String? getWsUrl() {
    var wsHost = const String.fromEnvironment('WS_HOST');
    if (wsHost.isEmpty && kIsWeb) {
      throw Exception(
          'You must define WS_HOST. This can be done with the --dart-define arg to run or build');
    }
    return kIsWeb ? wsHost : Platform.environment['WS_HOST'];
  }

  String? get authHeader {
    var remoteApiKey = const String.fromEnvironment('REMOTE_API_KEY');
    if (remoteApiKey.isEmpty && kIsWeb) {
      throw Exception(
          'You must define REMOTE_API_KEY. This can be done with the --dart-define arg to run or build');
    }
    return kIsWeb ? remoteApiKey : Platform.environment['REMOTE_API_KEY'];
  }

  String? get secret {
    var secretKey = const String.fromEnvironment('SECRET_KEY');
    if (secretKey.isEmpty && kIsWeb) {
      throw Exception(
          'You must define SECRET_KEY. This can be done with the --dart-define arg to run or build');
    }
    return kIsWeb ? secretKey : Platform.environment['SECRET_KEY'];
  }

  void setToken(String authToken) {
    _token = authToken;
  }

  String? get token => _token;
}
