import 'package:flutter_test/flutter_test.dart';
import 'package:sendbird_sdk/request/abstract/api_request.dart';
import 'package:sendbird_sdk/sendbird_sdk.dart';

import 'dart:async';

import 'package:shared_preferences/shared_preferences.dart';
import 'package:universal_io/io.dart' as io;

import 'common.dart';
import 'helpers/private_api_client.dart';
import 'request.dart';

class TestAppInfo {
  String appId;
  String token;

  TestAppInfo(this.appId, this.token);
}

typedef FutureGenerator<T> = Future<T> Function(ApiRequest);
Future<T> retry<T>(FutureGenerator aFuture, ApiRequest req,
    {required int retries, required Duration delay}) async {
  try {
    return await aFuture(req);
  } catch (e) {
    if (retries > 1) {
      await Future.delayed(delay);
      return retry(aFuture, req, retries: retries - 1, delay: delay);
    }
    rethrow;
  }
}

Future<TestAppInfo> testCreateApp() async {
  final apiHost = env.getApiUrl();
  // dep
  final api = PrivateApiClient();
  api.initialize(baseUrl: apiHost);
  String appId, token;

  final res = await retry<Map>(
    api.send,
    AppCreateRequest(),
    retries: 3,
    delay: Duration(seconds: 3),
  );
  appId = res['app_id'];
  token = res['api_token'];
  env.setToken(token);

  // dep
  api.initialize(token: token);

  await api.send(AppPlanAttributesUpdateRequest(appId, {
    'plan': 'enterprise',
    'translation_vendor': 'bing',
    'push_enabled': true,
    'use_new_ops': true,
    'unread_cnt_subscription_all': true,
    'use_group_channel_changelogs': true,
    'unread_cnt_subscription_custom_type': [
      'CUSTOM_TYPE_A',
      'CUSTOM_TYPE_B',
      'CUSTOM_TYPE_C'
    ],
    'plan_premiums': [
      'delivery_receipt',
      'disappearing_messages',
      'message_search_v3',
      'poll'
    ],
  }));

  await api.send(AppReactionEnableRequest(appId, true));
  await api.send(AppAttributesUpdateRequest(appId, {
    'acl': {
      'allow_super_group_channel': true,
      'allow_broadcast_channel': true,
    }
  }));

  return TestAppInfo(appId, token);
}

Future<SendbirdSdk> testApiSdkSetup({required TestAppInfo testInfo}) async {
  TestWidgetsFlutterBinding.ensureInitialized();
  SharedPreferences.setMockInitialValues({});
  io.HttpOverrides.global = null;

  final sdk = SendbirdSdk(appId: testInfo.appId, apiToken: testInfo.token);
  return sdk;
}

Future<void> testApiSdkCleanup(SendbirdSdk sdk) async {
  await sdk.disconnect();
}
