import 'dart:async';
import 'package:sendbird_sdk/sendbird_sdk.dart';
import 'package:sendbird_sdk/sdk/sendbird_sdk_api.dart';

import 'common.dart';
import 'helpers/private_api_client.dart';

//NOTE: setup sdk should call prior to call this method
Future<User> testApiCreateUser({
  String userId = '',
  String nickname = '',
  String profileUrl = '',
  bool accessToken = false,
  bool issueSessionToken = false,
  int? sessionTokenExpiresAt,
}) async {
  final api = PrivateApiClient();
  final user = await api.createUser(
    userId: userId,
    nickname: nickname,
    profileUrl: profileUrl,
    accessToken: accessToken,
    issueSessionToken: issueSessionToken,
    sessionTokenExpiresAt: sessionTokenExpiresAt,
  );
  return user;
}

Future<List<User>> testApiCreateUsers(int count) async {
  var list = <User>[];

  for (var i = 0; i < count; i++) {
    final user = await testApiCreateUser();
    list.add(user);
  }
  return list;
}

Future<User> testApiCreateUserAndConnect() async {
  var user = await testApiCreateUser();
  try {
    user = await SendbirdSdk().connect(
      user.userId,
      apiHost: env.getApiUrl(),
      wsHost: env.getWsUrl(),
    );
  } catch (e) {
    rethrow;
  }

  return user;
}

Future<User> testApiConnect(String userId) async {
  if (userId.isEmpty) {
    throw InvalidParameterError();
  }
  try {
    var user = await SendbirdSdk().connect(
      userId,
      apiHost: env.getApiUrl(),
      wsHost: env.getWsUrl(),
    );
    return user;
  } catch (e) {
    rethrow;
  }
}

Future<List<User>> testApiCreateUsersAndConnect(int count) async {
  final users = await testApiCreateUsers(count);
  if (users.length != count) {
    throw UnknownError();
  }
  try {
    await SendbirdSdk().connect(
      users[0].userId,
      apiHost: env.getApiUrl(),
      wsHost: env.getWsUrl(),
    );
    return users;
  } catch (e) {
    rethrow;
  }
}
