//create websocket connection to sendbird server to test out multiple users

import 'dart:async';
import 'dart:convert';
import 'package:universal_io/io.dart';

import 'package:sendbird_sdk/sendbird_sdk.dart';
import 'package:sendbird_sdk/core/models/command.dart';
import 'package:sendbird_sdk/services/network/websocket_client.dart';
import 'package:sendbird_sdk/constant/contants.dart' as constants;
import 'package:sendbird_sdk/utils/parsers.dart';
import 'package:flutter/foundation.dart' show kIsWeb;

class WSTester {
  late WebSocketClient client;
  String userId;
  String appId;
  String wsHost;

  Map<String, Completer> completers = {};

  WSTester({required this.appId, required this.userId, required this.wsHost}) {
    client = WebSocketClient(
      onConnect: onWebSocketConnect,
      onDisconnect: onWebSocketDisconnect,
      onData: onWebSocketData,
      onError: onWebSocketError,
    );
  }

  Future<void> connect() async {
    var params = {
      'p': 'flutter',
      'pv': kIsWeb ? '' : Platform.version.split(' ').first,
      if (kIsWeb == false) 'o': Platform.operatingSystem, // os
      if (kIsWeb == false) 'ov': Platform.operatingSystemVersion, // os version
      'sv': '3.0.13',
      'ai': appId,
      'user_id': userId,
      'SB-User-Agent': 'flutter/c3.0.0',
      'include_extra_data': [
        constants.sbExtraDataPremiumFeatureList,
        constants.sbExtraDataFileUploadSizeLimit,
        constants.sbExtraDataApplicationAttributes,
        constants.sbExtraDataEmojiHash,
      ].join(','),
      'expiring_session': '0',
    };

    final fullWsHost = wsHost + '/?' + Uri(queryParameters: params).query;
    await client.connect(fullWsHost);
    await Future.delayed(Duration(seconds: 2));
  }

  Future<bool> close() async {
    return client.close();
  }

  Future<Command?> sendCommand(Command cmd) async {
    try {
      client.send(cmd.encode());
    } catch (e) {
      rethrow;
    }

    if (cmd.isAckRequired) {
      final completer = Completer<Command>();
      completers[cmd.requestId!] = completer;
      return completer.future;
    } else {
      return null;
    }
  }

  void onWebSocketConnect() {}
  void onWebSocketDisconnect() {}
  void onWebSocketData(dynamic data) async {
    String stringCommand;

    if (data.runtimeType == String) {
      stringCommand = (data as String);
    } else {
      stringCommand =
          Utf8Decoder(allowMalformed: true).convert(data as List<int>);
    }

    if (stringCommand.isEmpty) {
      return;
    }

    final cmd = await parseCommand(stringCommand);
    completers.remove(cmd.requestId)?.complete(cmd);
  }

  void onWebSocketError(Object error) {}

  Future<void> enter(BaseChannel channel) async {
    final cmd = Command.buildEnterChannel(channel);
    await sendCommand(cmd);
  }

  Future<void> exit(BaseChannel channel) async {
    final cmd = Command.buildExitChannel(channel);
    await sendCommand(cmd);
  }

  void typing(GroupChannel channel, bool start) {
    final time = DateTime.now().millisecondsSinceEpoch;
    final startCmd = Command.buildStartTyping(channel.channelUrl, time);
    final endCmd = Command.buildEndTyping(channel.channelUrl, time);
    sendCommand(start ? startCmd : endCmd);
  }
}
