import 'package:flutter_test/flutter_test.dart';
import 'package:sendbird_sdk/sendbird_sdk.dart';

void main() {
  group('groupchannel eqaulity', () {
    test('should return correct boolean', () {
      final channel = GroupChannel(
        channelUrl: '123',
        createdAt: 123,
        customType: 'custom',
        isBroadcast: false,
        isStrict: true,
        isFrozen: false,
        isPublic: false,
        creator: null,
      );

      final channel2 = GroupChannel(
        channelUrl: '123',
        createdAt: 123,
        customType: 'custom',
        isBroadcast: false,
        isStrict: true,
        isFrozen: false,
        isPublic: false,
        creator: null,
      );

      expect(channel, equals(channel2));
      expect(channel, isNot(equals(channel2..customType = 'custom1')));
    });
  });

  group('openchannel eqaulity', () {
    test('should return correct boolean', () {
      final user = User(
        userId: 'op1',
        nickname: 'op1',
        connectionStatus: UserConnectionStatus.offline,
        metaData: {'aa': 'bb'},
        preferredLanguages: ['ko', 'en'],
      );

      final user2 = User(
        userId: 'op2',
        nickname: 'op2',
        connectionStatus: UserConnectionStatus.offline,
        metaData: {'aa': 'bb'},
        preferredLanguages: ['ko', 'en'],
      );

      final channel = OpenChannel(
        channelUrl: '123',
        createdAt: 123,
        customType: 'custom',
        isFrozen: false,
        creator: user,
        participantCount: 1,
        operators: [user, user2],
      );

      final channel2 = OpenChannel(
        channelUrl: '123',
        createdAt: 123,
        customType: 'custom',
        isFrozen: false,
        creator: user,
        participantCount: 1,
        operators: [user, user2],
      );

      expect(channel, equals(channel2));
      expect(channel, isNot(equals(channel2..creator = user2)));
    });
  });

  group('user eqaulity', () {
    test('should return correct boolean', () {
      final user = User(
        userId: 'user1',
        nickname: 'user1',
        connectionStatus: UserConnectionStatus.offline,
        metaData: {'aa': 'bb'},
        preferredLanguages: ['ko', 'en'],
      );

      final user2 = User(
        userId: 'user1',
        nickname: 'user1',
        connectionStatus: UserConnectionStatus.offline,
        metaData: {'aa': 'bb'},
        preferredLanguages: ['ko', 'en'],
      );

      expect(user, equals(user2));
      expect(user, isNot(equals(user2..userId = 'user2')));
      expect(
        user,
        isNot(equals(user2
          ..userId = 'user1'
          ..metaData = {'aa': 'cc'})),
      );
    });
  });

  group('member eqaulity', () {
    test('should return correct boolean', () {
      final member = Member(
        userId: 'user1',
        nickname: 'user1',
        connectionStatus: UserConnectionStatus.offline,
        metaData: {'aa': 'bb'},
        preferredLanguages: ['ko', 'en'],
        isMuted: true,
        state: MemberState.invited,
        isBlockedByMe: false,
        isBlockingMe: false,
      );

      final member2 = Member(
        userId: 'user1',
        nickname: 'user1',
        connectionStatus: UserConnectionStatus.offline,
        metaData: {'aa': 'bb'},
        preferredLanguages: ['ko', 'en'],
        isMuted: true,
        state: MemberState.invited,
        isBlockedByMe: false,
        isBlockingMe: false,
      );

      expect(member, equals(member2));
      expect(member, isNot(equals(member2..isMuted = false)));
    });
  });

  group('sender eqaulity', () {
    test('should return correct boolean', () {
      final sender = Sender(
        userId: 'user1',
        nickname: 'user1',
        connectionStatus: UserConnectionStatus.offline,
        metaData: {'aa': 'bb'},
        preferredLanguages: ['ko', 'en'],
        role: Role.chat_operator,
      );

      final sender2 = Sender(
        userId: 'user1',
        nickname: 'user1',
        connectionStatus: UserConnectionStatus.offline,
        metaData: {'aa': 'bb'},
        preferredLanguages: ['ko', 'en'],
        role: Role.chat_operator,
      );

      expect(sender, equals(sender2));
      expect(sender, isNot(equals(sender2..metaData = {'aa': 'cc'})));
    });
  });

  group('user message eqaulity', () {
    test('should return correct boolean', () {
      final message = UserMessage(
        channelType: ChannelType.group,
        channelUrl: 'channel_url',
        messageId: 1,
        message: 'message1',
        data: 'data',
        customType: 'custom',
        translations: {'es': 'ee'},
        sender: Sender(
          userId: 'user1',
          nickname: 'user1',
          connectionStatus: UserConnectionStatus.offline,
          metaData: {'aa': 'bb'},
          preferredLanguages: ['ko', 'en'],
        ),
      );

      final message2 = UserMessage(
        channelType: ChannelType.group,
        channelUrl: 'channel_url',
        message: 'message1',
        data: 'data',
        customType: 'custom',
        translations: {'es': 'ee'},
        messageId: 1,
        sender: Sender(
          userId: 'user1',
          nickname: 'user1',
          connectionStatus: UserConnectionStatus.offline,
          metaData: {'aa': 'bb'},
          preferredLanguages: ['ko', 'en'],
        ),
      );

      expect(message, equals(message2));
      expect(message, isNot(equals(message2..data = 'data1')));
    });
  });

  group('file message eqaulity', () {
    test('should return correct boolean', () {
      final message = FileMessage(
        url: 'http://naver.com',
        type: 'image/jpeg',
        messageId: 0,
        channelType: ChannelType.group,
        channelUrl: 'channel_url',
        threadInfo: ThreadInfo(replyCount: 1),
      );

      final message2 = FileMessage(
        url: 'http://naver.com',
        type: 'image/jpeg',
        messageId: 0,
        channelType: ChannelType.group,
        channelUrl: 'channel_url',
        threadInfo: ThreadInfo(replyCount: 1),
      );

      expect(message, equals(message2));
      expect(message,
          isNot(equals(message2..threadInfo = ThreadInfo(replyCount: 2))));
    });
  });

  group('thread info eqaulity', () {
    test('should return correct boolean', () {
      final thread = ThreadInfo(replyCount: 2, lastRepliedAt: 12);
      final thread2 = ThreadInfo(replyCount: 2, lastRepliedAt: 12);
      final thread3 = ThreadInfo(replyCount: 2, lastRepliedAt: 13);
      expect(thread, equals(thread2));
      expect(thread, isNot(equals(thread3)));
    });
  });

  group('reaction eqaulity', () {
    test('should return correct boolean', () {
      final reaction = Reaction(
        key: 'happy',
        userIds: ['user1', 'user2'],
        updatedAt: 12,
      );
      final reaction2 = Reaction(
        key: 'happy',
        userIds: ['user1', 'user2'],
        updatedAt: 12,
      );
      final reaction3 = Reaction(
        key: 'sad',
        userIds: ['user1', 'user2'],
        updatedAt: 12,
      );
      expect(reaction, equals(reaction2));
      expect(reaction, isNot(equals(reaction3)));
    });
  });
}
