import 'package:flutter_test/flutter_test.dart';

import 'package:sendbird_sdk/core/models/error.dart';
import 'package:sendbird_sdk/managers/session_manager.dart';
import 'package:shared_preferences/shared_preferences.dart';

void main() {
  TestWidgetsFlutterBinding.ensureInitialized();
  // Logger.level = Level.nothing;

  setUp(() {
    SharedPreferences.setMockInitialValues({});
  });

  group('session manager', () {
    test('should success init user session', () async {
      final session = SessionManager();
      session.setSessionKeyPath('com.sendbird.sdk.session');
      session.setUserIdKeyPath('com.sendbird.sdk.userid');

      session.setEKey('testEKey');
      session.setUserId('testUserIdisInttestvalue');
      await session.setSessionKey('testSessionKey');

      expect(session.getEKey(), 'testEKey');
      expect(session.getUserId(), 'testUserIdisInttestvalue');
      final key = await session.getSessionKey();
      expect(key, 'testSessionKey');
    });

    test('should throw an exception when user id is null', () async {
      final session = SessionManager();
      expect(() => session.setSessionKey('testSessionKey'),
          throwsA(isA<InvalidParameterError>()));
    });

    test('should return null on empty session', () async {
      final session = SessionManager();
      expect(await session.getSessionKey(), isNull);
    });

    test('should be instance class', () {
      final session = SessionManager();
      final session2 = SessionManager();
      expect(session != session2, isTrue);
    });
  });
}
