import 'package:flutter_test/flutter_test.dart';
import 'package:sendbird_sdk/sendbird_sdk.dart';

import '../common/sdksetup.dart';
import '../common/user.dart';

void main() {
  late TestAppInfo testInfo;
  late SendbirdSdk sdk;

  setUpAll(() async {
    testInfo = await testCreateApp();
  });

  setUp(() async {
    sdk = await testApiSdkSetup(testInfo: testInfo);
  });

  tearDown(() async {
    await sdk.disconnect();
  });

  tearDownAll(() async {
    //delete app?
  });

  group('openchannel list query', () {
    test('should return all created channels', () async {
      final user = await testApiCreateUserAndConnect();
      final params = OpenChannelParams()..operatorUserIds = [user.userId];

      await OpenChannel.createChannel(params);
      await OpenChannel.createChannel(params);
      await OpenChannel.createChannel(params);

      final query = OpenChannelListQuery();
      final res = await query.loadNext();
      expect(res.length, equals(3));
      expect(query.hasNext == false, isTrue);
    });

    test('should return limited channels with given limit', () async {
      await testApiCreateUserAndConnect();

      final query = OpenChannelListQuery();
      query.limit = 2;
      var res = await query.loadNext();
      expect(res.length, equals(2));
      expect(query.hasNext, isTrue);

      res = await query.loadNext();
      expect(res.length, equals(1));
      expect(query.hasNext, isFalse);
    });
    test('should not work without connection', () async {});
    test('should return along with metadata with includeMetadata option',
        () async {
      final user = await testApiCreateUserAndConnect();
      final params = OpenChannelParams()
        ..operatorUserIds = [user.userId]
        ..customType = 'custom 1';

      var channel = await OpenChannel.createChannel(params);
      await channel.createMetaData({'key1': 'value1'});
      channel = await OpenChannel.createChannel(params);
      await channel.createMetaData({'key2': 'value2'});

      final query = OpenChannelListQuery()
        ..includeMetaData = true
        ..customType = 'custom 1'
        ..limit = 5;

      final result = await query.loadNext();
      expect(result.length, equals(2));
      expect(result.first.getCachedMetaData(), isNotNull);
      expect(result.first.getCachedMetaData().length, equals(1));
      expect(result.last.getCachedMetaData(), isNotNull);
      expect(result.last.getCachedMetaData().length, equals(1));
    });

    test('should return json from toJson', () {
      final query = OpenChannelListQuery()
        ..channelUrl = '123'
        ..channelName = 'channel name'
        ..customType = 'custom'
        ..includeFrozenChannel = true
        ..limit = 15;

      final res = query.toJson();

      expect(res['channel_url'], equals('123'));
      expect(res['channel_name'], equals('channel name'));
      expect(res['custom_type'], equals('custom'));
      expect(res['include_frozen_channel'], isTrue);
      expect(res['limit'], equals(15));
    });
  });
}
