import 'package:flutter_test/flutter_test.dart';
import 'package:sendbird_sdk/sendbird_sdk.dart';

import '../common/sdksetup.dart';
import '../common/user.dart';

void main() {
  late TestAppInfo testInfo;
  late SendbirdSdk sdk;

  setUpAll(() async {
    testInfo = await testCreateApp();
  });

  setUp(() async {
    sdk = await testApiSdkSetup(testInfo: testInfo);
  });

  tearDown(() async {
    await sdk.disconnect();
  });

  tearDownAll(() async {
    //delete app?
  });

  group('operator list query', () {
    test('should return all operators in channel', () async {
      await testApiCreateUserAndConnect();
      final users = await testApiCreateUsers(3);
      final params = OpenChannelParams()
        ..operatorUserIds = users.map((e) => e.userId).toList();

      final channel = await OpenChannel.createChannel(params);
      final query = OperatorListQuery(
        channelType: channel.channelType,
        channelUrl: channel.channelUrl,
      );
      var result = await query.loadNext();
      expect(result, isNotNull);
      expect(result.length, equals(3));
      result = result.map((e) => e.userId).toList();
      users.forEach((user) => expect(result.contains(user.userId), isTrue));
    });

    test('should return limited operators with given limit', () async {
      await testApiCreateUserAndConnect();
      final users = await testApiCreateUsers(5);
      final params = OpenChannelParams()
        ..operatorUserIds = users.map((e) => e.userId).toList();

      final channel = await OpenChannel.createChannel(params);
      final query = OperatorListQuery(
        channelType: channel.channelType,
        channelUrl: channel.channelUrl,
      );
      query.limit = 3;

      var result = await query.loadNext();
      expect(result, isNotNull);
      expect(result.length, equals(3));
      expect(query.hasNext, isTrue);

      result = await query.loadNext();
      expect(result, isNotNull);
      expect(result.length, equals(2));
      expect(query.hasNext, isFalse);
    });

    test('should return json from toJson', () {
      final query = OperatorListQuery(
        channelType: ChannelType.group,
        channelUrl: '123',
      )..limit = 15;

      final res = query.toJson();

      expect(res['channel_url'], equals('123'));
      expect(res['channel_type'], equals('group'));
      expect(res['limit'], equals(15));
    });

    test('should not work without connection', () async {});
  });
}
